
--
--  Gossipmonger
--    by Tuhljin
--

local THIS_VERSION = GetAddOnMetadata("Gossipmonger", "Version");

local CYAN = "|cff00ffff"
local YELLOW = "|cffffff00"
local YELLOW_LT = "|cffffff99"
local R, G, B = 0.494, 0.729, 1
local SCTColors = { r = R, g = G, b = B }

local minimumGossipHeight, maximumGossipHeight = 150, 414
local incGossipHeight = 135

local ChatIcon = "|TInterface\\GossipFrame\\GossipGossipIcon:0:0:2:-2|t"

Gossipmonger = {};

Gossipmonger.DefaultSettings = {
  Version = THIS_VERSION, -- Safe to store with TjOptions-handled variables because it's static.
  skipIsDefault = true,
  informOnSkip_SCT = false,
  informOnSkip_log = false,
  alwaysInformWithName = false,
  showGossipMirrorOnSkip = true,
  delayShowingMirror = true,
  greetingModifierKey = 2,
};

local CurrentNPC, CurrentNPCIsBinder


-- LOCAL FUNCTIONS
-----------------------------------

local function chatprint(msg)
  DEFAULT_CHAT_FRAME:AddMessage(msg, R, G, B);
end

local function RegisterEvents_Show()
  Gossipmonger_MirrorFrame:RegisterEvent("BANKFRAME_OPENED");
  Gossipmonger_MirrorFrame:RegisterEvent("BATTLEFIELDS_SHOW");
  Gossipmonger_MirrorFrame:RegisterEvent("MERCHANT_SHOW");
  Gossipmonger_MirrorFrame:RegisterEvent("TAXIMAP_OPENED");
  Gossipmonger_MirrorFrame:RegisterEvent("TRAINER_SHOW");
  Gossipmonger_MirrorFrame:RegisterEvent("CONFIRM_BINDER");
end

local function RegisterEvents_Hide()
  Gossipmonger_MirrorFrame:RegisterEvent("BANKFRAME_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("BATTLEFIELDS_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("MERCHANT_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("TAXIMAP_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("TRAINER_CLOSED");
end

local function prepGossipOptionDisplay(titleButton, title, gossipType)
  titleButton:SetText(YELLOW..title.."   "..YELLOW_LT.."(selected)");
  GossipResize(titleButton);   -- Calls a Blizzard function that currently simply does this:  titleButton:SetHeight( titleButton:GetTextHeight() + 2);
  local titleButtonIcon = getglobal(titleButton:GetName() .. "GossipIcon")
  titleButtonIcon:SetTexture("Interface\\GossipFrame\\" .. gossipType .. "GossipIcon");
  titleButtonIcon:SetVertexColor(1, 1, 1, 1);
end

local function showGossipMirror(text, optionTitle, gossipType)
  Gossipmonger_MirrorFrame_GreetingText:SetText(text);
  prepGossipOptionDisplay(Gossipmonger_MirrorFrame_OptionDisplay, optionTitle, gossipType);

  local h = ( Gossipmonger_MirrorFrame_GreetingText:GetHeight() or 0 )
  if (h + incGossipHeight < minimumGossipHeight) then
    Gossipmonger_MirrorFrame:SetHeight(minimumGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Hide()
  elseif (h <= Gossipmonger_MirrorFrame_GreetingScrollFrame:GetHeight() and h + incGossipHeight < maximumGossipHeight) then
    Gossipmonger_MirrorFrame:SetHeight(h + incGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Hide()
  else
    Gossipmonger_MirrorFrame:SetHeight(maximumGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Show()
  end

  CurrentNPC = UnitName("npc");
  if (gossipType == "binder") then
    CurrentNPCIsBinder = true;
  else
    CurrentNPCIsBinder = nil;
  end
  local title = CYAN..( CurrentNPC or "Gossipmonger" )
  Gossipmonger_MirrorFrame_NpcNameText:SetText(title);
  if ( UnitExists("npc") ) then
    SetPortraitTexture(Gossipmonger_MirrorFrame_Portrait, "npc");
  else
    Gossipmonger_MirrorFrame_Portrait:SetTexture("Interface\\QuestFrame\\UI-QuestLog-BookIcon");
  end

  Gossipmonger_MirrorFrame_GreetingScrollFrame:SetVerticalScroll(0);
  if (not Gossipmonger_Settings.delayShowingMirror) then
    ShowUIPanel(Gossipmonger_MirrorFrame)
  end
  RegisterEvents_Show()
end

local function skipModifierKeyHeld()
  if (Gossipmonger_Settings.greetingModifierKey == 0) then
    return IsAltKeyDown();
  elseif (Gossipmonger_Settings.greetingModifierKey == 1) then
    return IsControlKeyDown();
  elseif (Gossipmonger_Settings.greetingModifierKey == 2) then
    return IsShiftKeyDown();
  end
  -- If key is set to None (3), it always returns nil.
end

local function skipType(gossipType)
  if (gossipType == "gossip") then  return false;  end    -- Never skip "gossip" type.
  -- Possible future settings could determine which types of dialog options to skip. For now, skips all the standard
  -- ones that WoW normally does.
  return true;
end

local function HideBubble()
  CurrentNPC = nil;
  CurrentNPCIsBinder = nil;
  if (Gossipmonger_MirrorFrame:IsShown()) then
    HideUIPanel(Gossipmonger_MirrorFrame)
  end
  Gossipmonger_MirrorFrame:UnregisterAllEvents();
end

local function SkipIsDefaultTooltip(self, tooltip)
  tooltip:AddLine(GOSSIPMONGER_STRINGS_SKIPISDEFAULT_TOOLTIP, nil, nil, nil, 1);
  tooltip:AddLine(GOSSIPMONGER_STRINGS_SKIPISDEFAULT_TOOLTIP2, nil, nil, nil, 1);
  tooltip:AddLine(GOSSIPMONGER_STRINGS_SKIPISDEFAULT_TOOLTIP3, nil, nil, nil, 1);
  tooltip:AddTexture("Interface\\GossipFrame\\GossipGossipIcon");
  tooltip:AddLine(GOSSIPMONGER_STRINGS_SKIPISDEFAULT_TOOLTIP4, nil, nil, nil, 1);
end

local function UpdateLabel()
  if (Gossipmonger_Settings.skipIsDefault) then
    Gossipmonger_GreetingModDropdown:SetLabel(GOSSIPMONGER_STRINGS_MODIFIERKEY_LABEL_DONOT)
  else
    Gossipmonger_GreetingModDropdown:SetLabel(GOSSIPMONGER_STRINGS_MODIFIERKEY_LABEL_DO)
  end
end

local function Init()
  Gossipmonger_MirrorFrame:SetScript("OnEvent", Gossipmonger.OnEvent);
  if (not Gossipmonger_Settings) then
    Gossipmonger_Settings = Gossipmonger.DefaultSettings;
  else
    Gossipmonger_Settings.Version = THIS_VERSION;
  end

  local KeySelectDropDown = {
    {
      text = GOSSIPMONGER_STRINGS_ALT..GOSSIPMONGER_STRINGS_KEY,
      tooltipTitle = GOSSIPMONGER_STRINGS_ALT..GOSSIPMONGER_STRINGS_KEY,
      tooltipText = GOSSIPMONGER_STRINGS_MODIFIERKEY_TOOLTIP_KEY:format(GOSSIPMONGER_STRINGS_ALT),
      value = 0
    },
    {
      text = GOSSIPMONGER_STRINGS_CTRL..GOSSIPMONGER_STRINGS_KEY,
      tooltipTitle = GOSSIPMONGER_STRINGS_CTRL..GOSSIPMONGER_STRINGS_KEY,
      tooltipText = GOSSIPMONGER_STRINGS_MODIFIERKEY_TOOLTIP_KEY:format(GOSSIPMONGER_STRINGS_CTRL),
      value = 1
    },
    {
      text = GOSSIPMONGER_STRINGS_SHIFT..GOSSIPMONGER_STRINGS_KEY,
      tooltipTitle = GOSSIPMONGER_STRINGS_SHIFT..GOSSIPMONGER_STRINGS_KEY,
      tooltipText = GOSSIPMONGER_STRINGS_MODIFIERKEY_TOOLTIP_KEY:format(GOSSIPMONGER_STRINGS_SHIFT),
      value = 2
    },
    {
      text = GOSSIPMONGER_STRINGS_MODIFIERKEY_NONE,
      tooltipTitle = GOSSIPMONGER_STRINGS_MODIFIERKEY_NONE,
      tooltipText = GOSSIPMONGER_STRINGS_MODIFIERKEY_NONE_TOOLTIP,
      value = 3
    };
  };

  local scttext = SCT and GOSSIPMONGER_STRINGS_INFORMONSKIP_SCT or GOSSIPMONGER_STRINGS_INFORMONSKIP_SCT_DISABLED;

  TjOptions.CreatePanel("Gossipmonger", nil, {
  	title = "Gossipmonger v"..THIS_VERSION,
  	items = {
  		 { variable = "skipIsDefault", text = GOSSIPMONGER_STRINGS_SKIPISDEFAULT,
  		   tooltip = SkipIsDefaultTooltip, OnChange = UpdateLabel },
  		 { variable = "greetingModifierKey", type = "dropdown", name = "Gossipmonger_GreetingModDropdown",
  		   text = GOSSIPMONGER_STRINGS_MODIFIERKEY_LABEL_DONOT, menu = KeySelectDropDown, width = 85 },
  		 { variable = "showGossipMirrorOnSkip", text = GOSSIPMONGER_STRINGS_SHOWBUBBLE },
  		 { variable = "delayShowingMirror", text = GOSSIPMONGER_STRINGS_SHOWBUBBLE_DELAY,
  		   tooltip = GOSSIPMONGER_STRINGS_SHOWBUBBLE_DELAY_TOOLTIP, xOffset = 14 },
  		 { type = "label", text = GOSSIPMONGER_STRINGS_INFORMONSKIP, xOffset = 0 },
  		 { variable = "informOnSkip_SCT", type = "checkboxsmall", text = scttext, xOffset = 14 },
  		 { variable = "informOnSkip_log", type = "checkboxsmall", text = GOSSIPMONGER_STRINGS_INFORMONSKIP_CHAT },
  		 { variable = "alwaysInformWithName", type = "checkboxsmall", text = GOSSIPMONGER_STRINGS_INFORMONSKIP_NAMEONLY,
  		   tooltip = GOSSIPMONGER_STRINGS_INFORMONSKIP_NAMEONLY_TOOLTIP },
  	},
  	variables = Gossipmonger_Settings,
  	defaults = Gossipmonger.DefaultSettings,
  	OnShow = UpdateLabel
  });

  Init = nil;
end


-- GLOBAL FUNCTIONS
-----------------------------------

function Gossipmonger.OnLoad()
  tinsert(UISpecialFrames, "Gossipmonger_MirrorFrame");
  UIPanelWindows.Gossipmonger_MirrorFrame = {area = "right", pushable = 8, whileDead = 1};
  Gossipmonger_MirrorFrame:SetScript("OnEvent", Init);
  Gossipmonger_MirrorFrame:RegisterEvent("VARIABLES_LOADED");
end

function Gossipmonger.OnEvent(self, event)
  if (event == "BANKFRAME_CLOSED" or event == "BATTLEFIELDS_CLOSED" or event == "MERCHANT_CLOSED" or
      event == "TAXIMAP_CLOSED" or event == "TRAINER_CLOSED") then
    HideBubble();
  elseif (CurrentNPC == UnitName("npc")) then
    if (not Gossipmonger_MirrorFrame:IsVisible()) then
      ShowUIPanel(Gossipmonger_MirrorFrame)
    end
    Gossipmonger_MirrorFrame:UnregisterAllEvents();
    RegisterEvents_Hide()
  else
    Gossipmonger_MirrorFrame:UnregisterAllEvents();
    CurrentNPC = nil;
    CurrentNPCIsBinder = nil;
  end
end

function Gossipmonger.GoodbyeClick()
  HideUIPanel(Gossipmonger_MirrorFrame);
  if (MerchantFrame:IsVisible()) then
    HideUIPanel(MerchantFrame)
  elseif (BankFrame:IsVisible()) then
    HideUIPanel(BankFrame)
  elseif (BattlefieldFrame:IsVisible()) then
    HideUIPanel(BattlefieldFrame)
  elseif (ArenaFrame:IsVisible()) then
    HideUIPanel(ArenaFrame)
  elseif (TaxiFrame:IsVisible()) then
    HideUIPanel(TaxiFrame)
  elseif (ClassTrainerFrame and ClassTrainerFrame:IsVisible()) then
    HideUIPanel(ClassTrainerFrame)
  end
  
  if (CurrentNPCIsBinder) then
    local dialog = StaticPopup_FindVisible("CONFIRM_BINDER");
    if (dialog) then
      StaticPopup_OnClick(dialog, 2)   -- Call as if user pressed cancel button
    end
  end
end


-- HOOK
-----------------

local orig_GossipFrame_OnEvent = GossipFrame_OnEvent;
GossipFrame_OnEvent = function(self, event, ...)
  if ( event == "GOSSIP_SHOW" and ( (GetNumGossipAvailableQuests() == 0) and (GetNumGossipActiveQuests() == 0) and (GetNumGossipOptions() == 1) ) ) then
-- See GossipFrame.lua in FrameXML.
    local optionTitle, gossipType = GetGossipOptions();

    -- Find instances where WoW normally skips gossip text.
    local keyheld = skipModifierKeyHeld()
    local skipthis = skipType(gossipType)
    if ( (Gossipmonger_Settings.skipIsDefault and skipthis and not keyheld) or (not Gossipmonger_Settings.skipIsDefault and skipthis and keyheld) ) then
      if (Gossipmonger_Settings.informOnSkip_SCT or Gossipmonger_Settings.informOnSkip_log) then
        local name = UnitName("npc")
        local title =  "The "..gossipType
        local text = " had something to say."
        local namelen = strlen(name)
        if ( Gossipmonger_Settings.alwaysInformWithName or (namelen <= 20 or namelen <= strlen (title)) ) then
          text = name..text
        else
          text = title..text
        end
        if (Gossipmonger_Settings.informOnSkip_SCT and SCT) then
          SCT:DisplayMessage( ChatIcon..text, SCTColors )
        end
        if (Gossipmonger_Settings.informOnSkip_log) then
          --chatprint(ChatIcon.." "..CYAN..text)  -- Unfortunately, 2.4.2 introduced a bug where displaying icons
                   -- like this results in distorted text in the chat log, so this feature won't be added yet.
          chatprint(CYAN..text)
        end
      end
      SelectGossipOption(1);
      if (Gossipmonger_Settings.showGossipMirrorOnSkip) then
        showGossipMirror(GetGossipText(), optionTitle, gossipType)
      end
      return;
    end

    if ( not GossipFrame:IsShown() ) then
      ShowUIPanel(self);
      if ( not self:IsShown() ) then
        CloseGossip();
        return;
      end
    end
    GossipFrameUpdate();

  else
    orig_GossipFrame_OnEvent(self, event, ...)
  end
end

local orig_binderOnHide = StaticPopupDialogs["CONFIRM_BINDER"].OnHide;
StaticPopupDialogs["CONFIRM_BINDER"].OnHide = function(...)
  HideBubble();
  if (orig_binderOnHide) then  orig_binderOnHide(...)  end
end
